/* IRCfs - IRC FileServ for *nix.
 * Copyright (C) 2002  Nick 'Zaf' Clifford <zaf@nrc.co.nz>
 * For licensing details, refer to the LICENSE file in the source 
 * code directory.
 */
#ifndef __include_event_h__
#define __include_event_h__

enum event_errors {
	event_err_mem=-1,
	event_err_not_event_obj=-2,
	event_err_invalid_event=-3,
	event_err_no_handlers=-4,
	event_err_eioio=-5, /* Event module brought the farm (big error) */
	event_err_not_object=-6,
	event_err_ok=0
};


enum event_filter_codes {
	event_filter_pass, /* Pass this event to the handler */
	event_filter_skip, /* Don't pass this event to the handler */
	event_filter_invalid /* This event is invalid */
};

enum event_handler_codes {
	event_handler_ok, /* The event handler handled this event, but
			   * continue calling other handlers */
	event_handler_passed, /* The event handler did not handle
				 this event, but continue calling other
				 handlers */
	event_handler_stop  /* Do not call any other event handlers for
			       this event. */
};   



int event_get_errno();


/* event_register_obj(obj,name)
 * Paramaters:
 * obj:
 * 	This is the object that the event object is to be associated
 * 	with.
 * Description:
 * 	This function sets up an object (see object.h) to be able
 * 	to raise and receive events.
 * Returns:
 * 	The error code on failure, or event_err_ok on success.
 * Error Codes:
 * 	event_err_mem: An internal memory problem
 * 	event_err_not_object: The handle passed doesn't refer to an
 * 		existing object.
 * 	event_err_already: The object has already been setup as
 * 		an event object.
 */ 	
int event_register_obj(HANDLE obj);

/* event_deregister_obj(obj)
 * Params:
 * obj:
 *      Specifies the object that is to loose its event object
 *      abilities.
 * Desc:
 * 	This function does the oppisite of event_register_obj()
 * 	It removes the code from the object that allows it to
 * 	raise and receive events. 
 * 	All listeners to this object and all callbacks in other event
 * 	objects from this object are are obviously removed.
 * Returns:
 * 	Returns the error code, or event_err_ok if no error.
 * Error Codes:
 * 	event_err_ok: No error
 * 	event_err_invalid_event_obj: The object passed isn't an 
 * 		event object.
 * 	event_err_mem: Some internal memory error occured.
 */
int event_deregister_obj(HANDLE obj);



/* event_add_handler(event_obj, handler_obj,
 * 	event_code, filter, 
 * 	handler_func, handler_data)
 * 
 * Parameters
 * ----------
 * event_obj:	
 * 	The event object to add the handler to.
 *
 * handler_obj:
 * 	The object to receive the event.
 * 	
 * event_code:
 * 	The event code to listen for. (See the discussion on event_codes)
 * 	
 * filter:
 * 	The event depedant filter data. (See the documentation for the
 * 	event to determine what type, if any, the filter should be.
 * 	
 * handler_func:
 * 	The function to call when the event_code is raised by the event_obj.
 * 	This function must be of the type event_callback_t.
 *
 * handler_data:
 * 	Extra data that is passed to the handler_func when the event is
 * 	raised.
 * 	
 * Description
 * -----------
 * 	Adds an event handler to the specified event object. The
 * 	handler is called when the specified event_code is raised by
 * 	the event object. If there is filter data specified, and the
 * 	event object has specified an event filter, then the event
 * 	filter is called first to determine if the specified 
 * 	handler_func should be called.
 * 	The handler_func is passed the event object, the handler object,
 * 	the event code,	the event dependant data (EDD), 
 * 	and the handler_data.
 *
 * Returns
 * -------
 * 	Returns event_err_ok on success, or an error code on failure.
 * 
 * Error Codes
 * -----------
 *  	event_err_not_event_obj: Either the event_obj or the handler_obj
 *  		are not an event object (eg it has not be registered 
 *  		with event_register first).
 *  	event_err_invalid_callback: The specified callback is not
 *  		valid.
 *  	event_err_bad_filter: The filter specified is not in the
 *  		correct format (did not pass the event_check_filter
 *  		function of the event object).
 *  	event_err_invalid_event: The event code specified is not
 *  		a valid event code.
 */

int event_add_handler(HANDLE obj, HANDLE handler_obj,
		int event_code,
		void *filter, /* Event dependant filter */
		event_callback_t handler_func,
		void *handler_data);

/* Delete all events in a specified event object */
int event_del_handler_by_event_obj(HANDLE event_obj);
/* Delete all events in a specified event object that have the specified
 * event code 
 */
int event_del_handler_by_event_code(HANDLE event_obj, int event_code);
/* Delete all callbacks setup by the specified handler_obj
 */
int event_del_handler_by_handler_obj(HANDLE handler_obj); 

/* Delete a specified callback, specified by event_obj, handler_obj,
 * and event_code
 */
int event_del_handler(HANDLE event_obj, HANDLE handler_obj,
		int event_code);



int event_raise(HANDLE event_obj, int event_code, void *event_edd);

/* Sets the event filter for the event object */
int event_set_filter(HANDLE event_obj, event_filter_func_t func);

int event_set_check_filter(HANDLE event_obj, 
		event_filter_check_func func);



#endif
